/*********************************************************************
* Filename:   aes_test.c
* Author:     Brad Conte (brad AT bradconte.com)
* Copyright:
* Disclaimer: This code is presented "as is" without any guarantees.
* Details:    Performs known-answer tests on the corresponding AES
              implementation. These tests do not encompass the full
              range of available test vectors and are not sufficient
              for FIPS-140 certification. However, if the tests pass
              it is very, very likely that the code is correct and was
              compiled properly. This code also serves as
	          example usage of the functions.
*********************************************************************/

/* Modified for ECE4823 lab assignment by Yu-Cheng Chen
   Copyright (C) 2021 Georgia Institue of Technology
*/

/*************************** HEADER FILES ***************************/
#include <stdio.h>
#include <memory.h>
#include "aes.h"

/*********************** FUNCTION DEFINITIONS ***********************/
void print_hex(BYTE str[], int len)
{
	int idx;

	for(idx = 0; idx < len; idx++)
		printf("%02x", str[idx]);
}

int aes_ecb_test()
{
	WORD key_schedule[60], idx;
	BYTE enc_buf[128];

	BYTE plaintext[2][16] = {
		{0x32, 0x43, 0xf6, 0xa8, 0x88, 0x5a, 0x30, 0x8d, 0x31, 0x31, 0x98, 0xa2, 0xe0, 0x37, 0x07, 0x34}, //Plaintext1 
		{0x54, 0x68, 0x69, 0x73, 0x20, 0x69, 0x73, 0x20, 0x61, 0x20, 0x73, 0x65, 0x63, 0x72, 0x65, 0x74	}  //Plaintext2
	};
	BYTE ciphertext[2][16] = {
		{0x39, 0x25, 0x84, 0x1d, 0x02, 0xdc, 0x09, 0xfb, 0xdc, 0x11, 0x85, 0x97, 0x19, 0x6a, 0x0b, 0x32}, //Ciphertext1 (expected ciphertext corresponding to plaintext 1)
		{0x65, 0xbf, 0x63, 0xdf, 0x76, 0x87, 0xd1, 0xc1, 0xb3, 0x8e, 0xda, 0x29, 0xd4, 0x16, 0x66, 0x6d}  //Ciphertext2 (expected ciphertext corresponding to plaintext 2)
	};

	BYTE key[1][16] = {
		{0x2b, 0x7e, 0x15, 0x16, 0x28, 0xae, 0xd2, 0xa6, 0xab, 0xf7, 0x15, 0x88, 0x09, 0xcf, 0x4f, 0x3c} //128 bit key
	};

	int pass = 1;

	// Raw ECB mode.
	//printf("* ECB mode:\n");
	aes_key_setup(key[0], key_schedule, 128);
	//printf(  "Key          : ");
	//print_hex(key[0], 32);

	for(idx = 0; idx < 2; idx++) {
		aes_encrypt(plaintext[idx], enc_buf, key_schedule, 128);
		printf("\nPlaintext    : ");

		print_hex(plaintext[idx], 16);

		printf("\n-encrypted to: ");
		print_hex(enc_buf, 16);
		pass = pass && !memcmp(enc_buf, ciphertext[idx], 16);

		aes_decrypt(ciphertext[idx], enc_buf, key_schedule, 128);
		printf("\nCiphertext   : ");
		print_hex(ciphertext[idx], 16);
		printf("\n-decrypted to: ");
		print_hex(enc_buf, 16);
		pass = pass && !memcmp(enc_buf, plaintext[idx], 16);

		printf("\n\n");
	}

	return(pass);
}

int aes_test()
{
	int pass = 1;

	pass = pass && aes_ecb_test();
	return(pass);
}

int main(int argc, char *argv[])
{
	printf("AES Tests: %s\n", aes_test() ? "SUCCEEDED" : "FAILED");

	return(0);
}
